/* dcproto.h -- implementazione protocollo NMDC */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifndef DCLIBC_DCPROTO_H
#define DCLIBC_DCPROTO_H

#include <dclibc/userinfo.h>
#include <dclibc/hub.h>
#include <dclibc/search.h>

G_BEGIN_DECLS

/* struct per un messaggio */
typedef struct message_t
{
	char * nick;
	char * message;
} message_t;

/* struct per un redirect */
typedef struct redirect_t
{
	char * server;
	port_t port;
} redirect_t;

#define DC_DEFAULT_VERSION	"1.0091"
#define DC_SEARCH_SYNTAX	"%c?%c?%llu?%c?%s"	// size restricted, is max size, size, datatype, pattern

/* prefissi dei comandi */
#define DC_PREFIX_LOCK			"$Lock"
#define DC_PREFIX_KEY			"$Key"
#define DC_PREFIX_CHATMSG		"<"
#define DC_PREFIX_HUBNAME		"$HubName"
#define DC_PREFIX_HELLO			"$Hello"
#define DC_PREFIX_VERSION		"$Version"
#define DC_PREFIX_VALIDATENICK		"$ValidateNick"
#define DC_PREFIX_GETINFO		"$GetINFO"
#define DC_PREFIX_MYINFO		"$MyINFO"
#define DC_PREFIX_QUIT			"$Quit"
#define DC_PREFIX_TO			"$To:"
#define DC_PREFIX_GETPASS		"$GetPass"
#define DC_PREFIX_MYPASS		"$MyPass"
#define DC_PREFIX_BADPASS		"$BadPass"
#define DC_PREFIX_VALIDATEDENIDE	"$ValidateDenide"
#define DC_PREFIX_HUBISFULL		"$HubIsFull"
#define DC_PREFIX_LOGEDIN		"$LogedIn"
#define DC_PREFIX_KICK			"$Kick"
#define DC_PREFIX_CLOSE			"$Close"
#define DC_PREFIX_HUBTOPIC		"$HubTopic"
#define DC_PREFIX_GETNICKLIST		"$GetNickList"
#define DC_PREFIX_OPFORCEMOVE		"$OpForceMove"
#define DC_PREFIX_FORCEMOVE		"$ForceMove"
#define DC_PREFIX_SEARCH		"$Search"
#define DC_PREFIX_SR			"$SR"
#define DC_PREFIX_CONNECTTOME		"$ConnectToMe"
// comandi variabili
#define DC_PREFIX_NICKLIST		"$NickList"		// lista nick -- lista utenti
#define DC_PREFIX_OPLIST		"$OpList"		// lista nick -- lista operatori

/* sintassi dei comandi */
#define DC_CMD_LOCK		DC_PREFIX_LOCK " %s Pk=%s|"	// lock string, private/public key
#define DC_CMD_KEY		DC_PREFIX_KEY " %s|"		// key generata dalla lock
#define DC_CMD_CHATMSG		DC_PREFIX_CHATMSG "%s> %s|"	// mittente, messaggio -- messaggio in main chat
#define DC_CMD_HUBNAME		DC_PREFIX_HUBNAME " %s|"	// nome dell'hub
#define DC_CMD_HELLO		DC_PREFIX_HELLO " %s|"		// nick del nuovo utente entrato
#define DC_CMD_VERSION		DC_PREFIX_VERSION " %s|"	// versione del protocollo
#define DC_CMD_VALIDATENICK	DC_PREFIX_VALIDATENICK " %s|"	// nick -- valida il nick
#define DC_CMD_MYINFO		DC_PREFIX_MYINFO " %s %s %s<%s>$ $%s%c$%s$%llu$|"
				// destinatario (login: $ALL), nick, [client], tag, speed, type, email, sharesize
#define DC_CMD_GETINFO		DC_PREFIX_GETINFO " %s %s|"	// nick, mittente -- richiede info su un utente
#define DC_CMD_QUIT		DC_PREFIX_QUIT " %s|"		// nick -- utente uscito dall'hub
#define DC_CMD_TO		DC_PREFIX_TO " %s From: %s $<%s> %s|"
				// destinatario, mittente, mittente, messaggio -- messaggio privato
#define DC_CMD_GETPASS		DC_PREFIX_GETPASS "|"		// richiesta password
#define DC_CMD_MYPASS		DC_PREFIX_MYPASS " %s|"		// password -- invia la password
#define DC_CMD_BADPASS		DC_PREFIX_BADPASS "|"		// password errata
#define DC_CMD_VALIDATEDENIDE	DC_PREFIX_VALIDATEDENIDE " %s|"	// nick -- validazione nick negata
#define DC_CMD_HUBISFULL	DC_PREFIX_HUBISFULL "|"		// l'hub è pieno
#define DC_CMD_LOGEDIN		DC_PREFIX_LOGEDIN " %s|"	// nick -- operatore logginato
#define DC_CMD_KICK		DC_PREFIX_KICK " %s|"		// nick -- caccia un utente dall'hub
#define DC_CMD_CLOSE		DC_PREFIX_CLOSE " %s|"		// nick -- disconnette un utente dall'hub
#define DC_CMD_HUBTOPIC		DC_PREFIX_HUBTOPIC " %s|"	// topic dell'hub
#define DC_CMD_GETNICKLIST	DC_PREFIX_GETNICKLIST "|"	// richiede la lista utenti
#define DC_CMD_OPFORCEMOVE	DC_PREFIX_OPFORCEMOVE " $Who:%s$Where:%s$Msg:%s|"
				// vittima, server, motivazione -- redirige un utente su un altro hub
#define DC_CMD_FORCEMOVE	DC_PREFIX_FORCEMOVE " %s|"		// server -- redirezione forzata
#define DC_CMD_SEARCH(c)	({ char * __dummy; \
	if(c == TAG_MODE_ACTIVE) __dummy = DC_PREFIX_SEARCH " %s:%u %s|"; \
	else if(c == TAG_MODE_PASSIVE) __dummy = DC_PREFIX_SEARCH " Hub:%s %s|"; \
	__dummy; })
			// ricerca attiva: ip ricercatore, porta ricercatore, stringa di ricerca (DC_SEARCH_SYNTAX)
			// ricerca passiva: nick ricercatore, stringa di ricerca (DC_SEARCH_SYNTAX)
#define DC_CMD_CONNECTTOME	DC_PREFIX_CONNECTTOME " %s %s:%u|"
				// nick che richiede la connessione, host, porta -- richiesta di connessione al peer
#define DC_CMD_REVCONNECTTOME	DC_PREFIX_REVCONNECTTOME " %s %s|"
				// mittente, destinatario -- richiede una $ConnectToMe al peer (per passivi)

char * dc_proto_extract(const char * commands,unsigned int command_nr);
char * dc_proto_generate_key(char *lock);
char * dc_proto_content(const char * command);

gboolean dc_proto_parse(hub_t * hub);
char * dc_proto_parse_lock(hub_t * hub,const char * command);
char * dc_proto_parse_hello(hub_t * hub,const char * command,char ** nick);
void dc_proto_parse_hubname(hub_t * hub,const char * command);
char * dc_proto_parse_quit(hub_t * hub,const char * command);
void dc_proto_parse_to(const char * command,message_t * msg);
char * dc_proto_parse_logedin(hub_t * hub,const char * command);
void dc_proto_parse_hubtopic(hub_t * hub,const char * command);
char * dc_proto_parse_myinfo(hub_t * hub,const char * command);
void dc_proto_parse_nicklist(hub_t * hub,const char * command);
void dc_proto_parse_oplist(hub_t * hub,const char * command);
redirect_t * dc_proto_parse_forcemove(hub_t * hub,const char * command);
search_t * dc_proto_parse_search(const char * command);
result_t * dc_proto_parse_sr(const char * command);

gboolean dc_proto_parse_message(const char * message,message_t * buffer);
char * dc_proto_extract_user(const char * buf);
char * dc_proto_extract_message(const char * buf);
char * dc_proto_convert_reserved(const char * str);
char * dc_proto_deconvert_reserved(const char * str);

int dc_proto_send(hub_t * hub,const char * command,...);
void dc_proto_send_main_message(hub_t * hub,const char * message);
void dc_proto_send_priv_message(hub_t * hub,const char * nick,const char * message);
void dc_proto_kick(hub_t * hub,const char * nick);
void dc_proto_close(hub_t * hub,const char * nick);
void dc_proto_send_password(hub_t * hub,const char * pass);
void dc_proto_quit(hub_t * hub);
void dc_proto_get_nicklist(hub_t * hub);
void dc_proto_get_info(hub_t * hub,const char * nick);
void dc_proto_opforcemove(hub_t * hub,const char * victim,const char * server,const char * reason);
int dc_proto_search(hub_t * hub,gboolean size_restricted,gboolean is_max_size,guint64 size,DATA_TYPE type,const char * pattern);

G_END_DECLS

#endif	/* DCLIBC_DCPROTO_H */
