/* hub.h -- definizioni comuni hub */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifndef DCLIBC_HUB_H
#define DCLIBC_HUB_H

#include <dclibc/userinfo.h>
#include <dclibc/userlist.h>
#include <dclibc/hubconn.h>
#include <glib.h>

G_BEGIN_DECLS

/* descrittore hub */
typedef struct _hub_t
{
	HubConn * conn;		/* oggetto della connessione */
	char * hubserver;	/* server hub (solo store) */
	port_t hubport;		/* porta hub (solo store) */
	char * hubname;		/* nome hub */
	char * hubtopic;	/* topic hub */
	userinfo_t * userinfo;	/* info utente */
	userlist_t * userlist;	/* lista utenti */
	gpointer notify_func;	/* funzione di notifica per gli eventi dell'hub */
	GString * unparsed;	/* stringa con i dati non parsati */
	gboolean handshake;	/* indica se siamo in fase di login con l'hub */
	gint parsetag;		/* id per l'evento del parsing */
	GQueue * notify_queue;
	gpointer user_data;	/* dati generici -- per l'utente */
	/* ricerca attiva */
	char * sr_host;		/* host sul quale mandare i pacchetti con i risultati */
	port_t sr_port;		/* porta UDP sulla quale mandare i pacchetti con i risultati */
	/* ricerca passiva */
	/* ricerca attiva e passiva */
	guint32 search_count;	/* numero di ricerche iniziate per l'hub */
} hub_t;

typedef void (*HubNotifyFunc)(hub_t * hub,gint notify_type,gpointer data);

//#define HUB_NOTIFY(hub,notify,data)	((HubNotifyFunc)hub->notify_func)(hub,notify,data)
void HUB_NOTIFY(hub_t * hub,gint notify_type,gpointer data,GDestroyNotify destroy_data);

/* notifiche */
#define NOTIFY_SYSERROR		-2	// errore di sistema, data = NULL
#define NOTIFY_ERROR		-1	// errore generico, data = NULL
//					//
#define NOTIFY_DISCONNECTED	2	// disconnesso, data = NULL
#define NOTIFY_CONNECTING	3	// connessione in corso, data = NULL
#define NOTIFY_ERROR_RESOLV	4	// errore di risoluzione, data = NULL
#define NOTIFY_ERROR_CONNECT	5	// errore di connessione, data = NULL
#define NOTIFY_ERROR_REFUSED	6	// connessione rifiutata, data = NULL
#define NOTIFY_ERROR_TIMEOUT	7	// timeout di connessione, data = NULL
#define NOTIFY_CONNECTED	8	// connesso, data = NULL
#define NOTIFY_JOIN		9	// utente logginato, data = nick
#define NOTIFY_ASKPASS		10	// richiesta della password, data = NULL
#define NOTIFY_HUBNAME		11	// aggiornamento nome hub, data = hubname
#define NOTIFY_PART		12	// utente uscito, data = nick
#define NOTIFY_CHATMSG		13	// messaggio in main, data = struct message_t { <nick>, <messaggio> };
#define NOTIFY_PRIVMSG		14	// messaggio privato, data = struct message_t { <nick>, <messaggio> };
#define NOTIFY_HUBFULL		15	// hub pieno, data = NULL
#define NOTIFY_INVALIDNICK	16	// nick non validato, data = NULL
#define NOTIFY_BADPASS		17	// password errata, data = NULL
#define NOTIFY_REDIRECT		18	// redirezione forzata, data = struct redirect_t { <server>, <porta> };
#define NOTIFY_OPJOIN		19	// un op è entrato in hub ($LogedIn), data = nick
#define NOTIFY_HUBTOPIC		20	// aggiornamento topic hub, data = topic
#define NOTIFY_USERINFO		21	// informazioni su un utente, data = nick
#define NOTIFY_NICKLIST		22	// lista utenti, data = NULL (controllare nella userlist dell'hub)
#define NOTIFY_OPLIST		23	// lista operatori, data = NULL (controllare nella userlist dell'hub)
#define NOTIFY_SEARCH_REQUEST	24	// richiesta di ricerca, data = struct search_t
#define NOTIFY_SEARCH_RESULT	25	// risultato di ricerca, data = struct result_t

hub_t * dc_hub_new(const char * server,port_t port,userinfo_t * userinfo,userlist_t * userlist,HubNotifyFunc notify_func,gpointer user_data);
void dc_hub_delete(hub_t * hub);
gboolean dc_hub_connect(hub_t * hub);
void dc_hub_disconnect(hub_t * hub);

G_END_DECLS

#endif	/* DCLIBC_HUB_H */
