/* hubconn.h -- gestisce la connessione ad un hub */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifndef DCLIBC_HUBCONN_H
#define DCLIBC_HUBCONN_H

#include <sys/types.h>

G_BEGIN_DECLS

typedef unsigned int port_t;

typedef struct _HubConn
{
	int sock_fd;	// descrittore socket
	char * server;	// server remoto
	char * serverip;// ip del server remoto
	port_t port;	// porta remota
	pid_t pid;	// pid del processo di connessione
	int pipes[2];	// pipe per comunicare con il figlio ([0] lettura, [1] scrittura)
	gint socktag;	// sourceid per gli eventi della socket
	gint iotag;	// sourceid per l'evento di dati in arrivo da pipes[0]
	gint timetag;	// sourceid per l'evento del timeout di connessione
	gint outtag;	// sourceid per l'evento flush dei dati in uscita
	gboolean connected;	// flag che indica se siamo connessi o meno
	gpointer notifier;	// funzione per le notifiche
	GString * buffer;	// buffer di input
	GString * out_queue;	// coda dati in uscita
	gpointer user_data;	// dati per l'utente
} HubConn;

typedef enum
{
	CONN_NOTIFY_SYSERROR = -2,	// errore di sistema, data = NULL
	CONN_NOTIFY_ERROR = -1,		// errore generico, data = NULL
	CONN_NOTIFY_NONE = 0,		// niente :P, data = NULL
	CONN_NOTIFY_CONNECTING,		// procedura di connessione in procinto di iniziare, data = NULL
	CONN_NOTIFY_DISCONNECTED,	// disconnesso, data = NULL
	CONN_NOTIFY_ERROR_RESOLV,	// errore di risoluzione, data = NULL
	CONN_NOTIFY_ERROR_CONNECT,	// errore di connessione, data = NULL
	CONN_NOTIFY_ERROR_REFUSED,	// connessione rifiutata, data = NULL
	CONN_NOTIFY_ERROR_TIMEOUT,	// timeout di connessione, data = NULL
	CONN_NOTIFY_RESOLVED,		// indirizzo risolto, data = NULL
	CONN_NOTIFY_CONNECTED,		// connesso, data = NULL
	CONN_NOTIFY_DATA		// dati in arrivo, data = NULL
} HUBCONN_NOTIFY_TYPES;

typedef void (*HubConnNotifier)(HubConn * conn,HUBCONN_NOTIFY_TYPES notify_type,gpointer data);

#define HUBCONN_NOTIFY(conn,notify,data)	((HubConnNotifier)conn->notifier)(conn,notify,data)

void dc_hub_conn_init(void);
HubConn * dc_hub_conn_new(const char * server,port_t port,HubConnNotifier cb_notify,gpointer user_data);
void dc_hub_conn_delete(HubConn * conn);
void dc_hub_conn_open(HubConn * conn);
void dc_hub_conn_close(HubConn * conn);
void dc_hub_conn_append_data(HubConn * conn,char * buf);

G_END_DECLS

#endif	/* DCLIBC_HUBCONN_H */
