/* search.h -- sistema di ricerca */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifndef DCLIBC_SEARCH_H
#define DCLIBC_SEARCH_H

#include <dclibc/hub.h>

/* File types:
 1 for any file type
 2 for audio files ("mp3", "mp2", "wav", "au", "rm", "mid", "sm")
 3 for compressed files ("zip", "arj", "rar", "lzh", "gz", "z", "arc", "pak")
 4 for document ("doc", "txt", "wri", "pdf", "ps", "tex")
 5 for executable ("pm", "exe", "bat", "com")
 6 for picture ("gif", "jpg", "jpeg", "bmp", "pcx", "png", "wmf", "psd")
 7 for video ("mpg", "mpeg", "avi", "asf", "mov")
 8 for folder
 9 for TTH hash
*/
typedef enum DATA_TYPE
{
	DATA_ANY = '1',
	DATA_AUDIO = '2',
	DATA_COMPRESSED = '3',
	DATA_DOCUMENT = '4',
	DATA_EXECUTABLE = '5',
	DATA_PICTURE = '6',
	DATA_VIDEO = '7',
	DATA_FOLDER = '8',
	DATA_TTH = '9'
} DATA_TYPE;

/* Search Result */
typedef struct _result_t
{
	char * nick;
	char * filename;
	unsigned long long filesize;
	unsigned int free_slots;
	unsigned int total_slots;
	struct
	{
		char * hubname;
		char * tth;
	} extendedprotocol;
	char * hubhost;
	port_t hubport;
	gboolean is_folder;
} result_t;

/* Search Request */
typedef struct _search_t
{
	char * host_nick;		// attivo: host/ip del ricercatore, passivo: nick del ricercatore
	port_t port;			// porta del ricercatore (se passivo: 0)
	gboolean size_restricted;	// indica se la ricerca deve essere ristretta per dimensione
	gboolean is_max_size;		// indica se il successivo parametro size e` la dimensione massima
	guint64 size;			// dimensione alla quale e` ristretta la ricerca
	DATA_TYPE type;			// tipo di file
	char * pattern;			// pattern di ricerca
} search_t;

void dc_search_init(void);
int dc_search_request(hub_t * hub,gboolean size_restricted,gboolean is_max_size,guint64 size,DATA_TYPE type,const char * pattern);
void dc_search_cancel(hub_t * hub,guint32 count);
gboolean dc_search_set_active_search_params(hub_t * hub,const char * host,port_t port);

#endif	/* DCLIBC_SEARCH_H */
