/* userinfo.h -- gestione info utente */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifndef DCLIBC_USERINFO_H
#define DCLIBC_USERINFO_H

#include <glib.h>

G_BEGIN_DECLS

#define TAG_MODE_ACTIVE		'A'
#define TAG_MODE_PASSIVE	'P'

/* info utente */
typedef struct _userinfo_t
{
	char * nick;		// nick
	char * email;		// e-mail
	char * client;		// [optional] identificativo del client (puo` non esserci)
	char * desc;		// [optional] descrizione (puo` non esserci)
	char * tag;		// tag (senza < >)
	char * speed;		// velocità (DSL, LAN(T1), ecc.)
	char type;		// tipo di client
	gboolean is_shared;	// se TRUE allora share_size e` valido
	unsigned long long share_size;	// quantita` di byte condivisi
	gboolean is_op;		// e` un operatore?
} userinfo_t;

/* velocità del client */
typedef enum SPEED
{
	MODEM = 0,	// modem generico
	KBPS288,	// modem 28.8kbps, antico senza compressione
	KBPS336,	// modem 56kbps senza compressione
	KBPS56,		// model 56kpbs con compressione
	SATELLITE,	// connessione in ricezione satellitare
	ISDN,		// linea isdn
	DSL,		// linea adsl
	CABLE,		// ???
	LANT1,		// hdsl o adsl >= 5 mbit/s
	LANT3,		// fibra ottica o connessione diretta
	NUM_SPEEDS
} SPEED;

/* tipo del client */
typedef enum CLIENT_TYPE
{
	NORMAL = 1,
	NORMAL_AWAY1,
	NORMAL_AWAY2,
	SERVER1,
	SERVER2,
	SERVER_AWAY1,
	SERVER_AWAY2,
	FIREBALL1,
	FIREBALL2,
	FIREBALL_AWAY1,
	FIREBALL_AWAY2,
	NUM_CLIENTS
} CLIENT_TYPE;

#define userinfo_get_share_size(x)	(x->share_size)
#define userinfo_get_type(x)		(x->type)
#define userinfo_is_op(x)		(x->is_op)
#define userinfo_get_tag(x)		(x->tag)
#define userinfo_get_nick(x)		(x->nick)
#define userinfo_get_email(x)		(x->email)
#define userinfo_get_client(x)		(x->client)
char userinfo_get_mode(userinfo_t * userinfo);

userinfo_t * userinfo_new(const char * nick,const char * email,const char * desc,const char * client_name,SPEED speed,CLIENT_TYPE type,gboolean is_shared,unsigned long long share_size);
void userinfo_delete(userinfo_t * userinfo);
void userinfo_set_tag(userinfo_t * userinfo,const char * client_name,const char * client_version,guint8 mode,guint hubs_normal,guint hubs_reg_vip,guint hubs_op,guint slots,guint speed_below_slot);
userinfo_t * userinfo_merge(userinfo_t * u1,userinfo_t * u2);
userinfo_t * userinfo_copy(userinfo_t * u);

G_END_DECLS

#endif	/* DCLIBC_USERINFO_H */
